package com.nemrodsoftware;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.List;

import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.nemrodsoftware.model.Locale;
import com.nemrodsoftware.model.Module;
import com.nemrodsoftware.model.Project;
import com.nemrodsoftware.model.StatusMessage;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.GenericType;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.json.JSONConfiguration;
import com.sun.jersey.client.apache.ApacheHttpClient;
import com.sun.jersey.client.apache.config.ApacheHttpClientConfig;
import com.sun.jersey.client.apache.config.DefaultApacheHttpClientConfig;
import com.sun.jersey.core.header.ContentDisposition;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import com.sun.jersey.multipart.FormDataMultiPart;
import com.sun.jersey.multipart.file.FileDataBodyPart;

public class NemrodClient implements INemrodClient
{
    private static final Log LOG = LogFactory.getLog(NemrodClient.class);
    
    private final String url;
    private final String login;
    private final String password;
    
    private final Client client ;
    private WebResource webResource;
    
    public NemrodClient(String url,String login,String password)
    {
        this.url = url;
        this.login = login;
        this.password = password;
        
        ClientConfig cc = new DefaultApacheHttpClientConfig();
        cc.getFeatures().put(JSONConfiguration.FEATURE_POJO_MAPPING, Boolean.TRUE);
        cc.getProperties().put(ApacheHttpClientConfig.PROPERTY_HANDLE_COOKIES, Boolean.TRUE);
        client = ApacheHttpClient.create(cc);
        client.setFollowRedirects(true);
        webResource = client.resource(this.url);
    }
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#login()
     */
    @Override
    public boolean login()
    {
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
        queryParams.add("username", login);
        queryParams.add("password", password);
        
        ClientResponse response = webResource.queryParams(queryParams)
                                       .path("/login")
                                       .accept(MediaType.APPLICATION_JSON)
                                       .post(ClientResponse.class);

        StatusMessage msg = response.getEntity(StatusMessage.class);
        
        if (!msg.getMessage().equals("OK"))
        {
            LOG.error("Error : "+msg.getErrors());
            return false;
        }
        return true;
    }

    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#findProject(java.lang.String)
     */
    @Override
    public Project findProject(String projectName)
    {
        List<Project> projects = listProject();
        for (Project project : projects)
        {
            if (project.getName().equals(projectName))
                return project;
        }
        return null;
    }
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#listProject()
     */
    @Override
    public List<Project>listProject()
    {
        GenericType<List<Project>> genericType = new GenericType<List<Project>>() {};
        
        List<Project> projects = webResource.path("/projects/index")
                   .accept(MediaType.APPLICATION_JSON)
                   .get(genericType);
        return projects;
    }
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#listLocale(com.nemrodsoftware.model.Project)
     */
    @Override
    public List<Locale>listLocale(Project project)
    {
        GenericType<List<Locale>> genericType = new GenericType<List<Locale>>() {};
        
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
        queryParams.add("projectId", project.getId().toString());
        
        List<Locale> locales = webResource.queryParams(queryParams)
                   .path("/projects/locales")
                   .accept(MediaType.APPLICATION_JSON)
                   .get(genericType);
        return locales;
    }
    

    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#findLocale(com.nemrodsoftware.model.Project, java.lang.String)
     */
    @Override
    public Locale findLocale(Project project, String localeName)
    {
        List<Locale> locales = listLocale(project);
        for (Locale locale : locales)
        {
            if (localeName.equalsIgnoreCase( locale.getIsoName()))
                return locale;
        }
        return null;
    }    
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#listModule(com.nemrodsoftware.model.Project)
     */
    @Override
    public List<Module>listModule(Project project)
    {
        GenericType<List<Module>> genericType = new GenericType<List<Module>>() {};
        
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
        queryParams.add("projectId", project.getId().toString());
        
        List<Module> modules = webResource.queryParams(queryParams)
                   .path("/projects/modules")
                   .accept(MediaType.APPLICATION_JSON)
                   .get(genericType);
        return modules;
    }   
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#findModule(com.nemrodsoftware.model.Project, java.lang.String)
     */
    @Override
    public Module findModule(Project project, String moduleName, String modulePath)
    {
        List<Module> modules = listModule(project);
        for (Module module : modules)
        {
            if (module.getName().equals(moduleName) && module.getFullpath().equals(modulePath))
                return module;
        }
        return null;
    }     
    
    /* (non-Javadoc)
     * @see com.nemrodsoftware.INemrodClient#export(com.nemrodsoftware.model.Project, com.nemrodsoftware.model.Locale, com.nemrodsoftware.model.Module, java.lang.String, boolean, boolean)
     */
    @Override
    public File export(Project project, Locale locale, Module module, String exportType, boolean exportEmptyValues, boolean exportOnlyApprovedValues, String workFolder) throws IOException
    {
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
        queryParams.add("projectId", project.getId().toString());
        queryParams.add("localeId", locale.getId().toString());
        queryParams.add("moduleId", module.getId().toString());
        queryParams.add("exporter", exportType);
        queryParams.add("exportemptyvalues", Boolean.toString(exportEmptyValues));
        queryParams.add("onlyapprovedvalues", Boolean.toString(exportOnlyApprovedValues));
        
        ClientResponse response = webResource.queryParams(queryParams)
                    .path("/fileexporters/doexport")
                    .accept(MediaType.APPLICATION_OCTET_STREAM)
                    .get(ClientResponse.class);

        String header = response.getHeaders().getFirst("Content-Disposition");
        ContentDisposition cd;
        try
        {
            cd = new ContentDisposition(header);
        } catch (ParseException e)
        {
            throw new IllegalStateException("header Content-disposition not found or invalid. Header found="+header,e);
        }
        String filename = cd.getFileName();
        InputStream is = response.getEntityInputStream();
        
        File dest = new File (workFolder,filename);
        
        FileUtils.writeByteArrayToFile(dest, IOUtils.toByteArray(is));
        
        return dest;        
    }

    @Override
    public boolean sync(Project project, String format, Module module, File masterFile)
    {
        MultivaluedMap<String, String> queryParams = new MultivaluedMapImpl();
        queryParams.add("projectId", project.getId().toString());
        queryParams.add("importer", format);
        queryParams.add("moduleId", module.getId().toString());
        
        FormDataMultiPart form = new FormDataMultiPart();
        form.field("projectId", project.getId().toString());
        form.field("importer", format);
        form.field("moduleId", module.getId().toString());
        //InputStream is = IOUtils.copy(FileUtils.openInputStream(masterFile));
        form.bodyPart(new FileDataBodyPart("masterFile", masterFile,  MediaType.APPLICATION_OCTET_STREAM_TYPE));
        //form.field("masterFile", is , MediaType.APPLICATION_OCTET_STREAM_TYPE);
       
        ClientResponse response = webResource.path("/projects/sync")
        .type(MediaType.MULTIPART_FORM_DATA_TYPE)
        .accept(MediaType.APPLICATION_JSON)
        .post(ClientResponse.class,form);
        
        StatusMessage msg = response.getEntity(StatusMessage.class);
        
        if (!msg.getMessage().equals("OK"))
        {
            LOG.error("Error : "+msg.getDetail());
            return false;
        }
        return true;
    }

}
